package ru.ifmo.ihouse.doorlock;

import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.Timer;
import java.util.TimerTask;

import ru.ifmo.ihouse.core.Device;
import ru.ifmo.ihouse.core.Message;
import ru.ifmo.ihouse.events.SystemEventProvider;

import com.evelopers.common.exception.CommonException;
import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.runtime.ControlledObject;
import com.evelopers.unimod.runtime.EventProvider;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;

/**
 *   .     ,
 *     ,    .
 * @author rgm
 * @version 1.0.0
 */
public class DoorlockControl extends Device implements EventProvider {
	
	public DoorlockControl() {
		super();
	}
	
	public void launch() {
		InitAutomata("controls/ru/ifmo/ihouse/doorlock/A1.xml", this);
		start();
	}

	/**    .       */
	public final static long DOOR_OPENING_AWAITING_PERIOD = 5000;
	
	/**
	 * @unimod.event.descr  
	 */
	public static final String DL1 = "dl1";
	/**
	 * @unimod.event.descr  
	 */
	public static final String DL2 = "dl2";
	
	/**
	 * @unimod.event.descr  
	 */
	public static final String DL3 = "dl3";
	
	/**
	 * @unimod.event.descr   
	 */
	public static final String DL4 = "dl4";
	
	/**
	 * @unimod.event.descr   
	 */
	public static final String DL5 = "dl5";
	
	/**      */
	public String accCodes = "";
	/**    */
	private ArrayList<Long> _accCodes = new ArrayList<Long>();
	
	/**    */
	private Timer doorwaitTimer;
	
	/**
	 * @unimod.event.descr    
	 */
	public static final String DL6 = "dl6";
	
	/*
	 * (non-Javadoc)
	 * @see ru.ifmo.ihouse.core.Device#dispose()
	 */
	public void dispose() {}
	
	/*
	 * (non-Javadoc)
	 * @see com.evelopers.unimod.runtime.EventProvider#init(com.evelopers.unimod.runtime.ModelEngine)
	 */
	public void init(ModelEngine engine) throws CommonException {}

	/**
	 *   /
	 */
	private boolean codeAccepted = false;
	
	/**
	 * @unimod.action.descr   
	 */
	public void z1(StateMachineContext context) {
		TRACE(" . ");
		if (codeAccepted) engine.getEventManager().handle(new Event(DL2), StateMachineContextImpl.create());
		else engine.getEventManager().handle(new Event(DL3), StateMachineContextImpl.create());
	}
	
	/**
	 * @unimod.action.descr     
	 */
	public void z2(StateMachineContext context) {
		TRACE("  .  .");
	}
	
	/**
	 * @unimod.action.descr   
	 */
	public void z3(StateMachineContext context) {
		TRACE(" .  .");
	}
	
	/**
	 * @unimod.action.descr   
	 */
	public void z5(StateMachineContext context) {
		if (doorwaitTimer != null) doorwaitTimer.cancel();
		TRACE(" .  .");
	}
	
	/**
	 * @unimod.action.descr     
	 */
	public void z6(StateMachineContext context) {
		TRACE("   .   .");
	}
	
	/**
	 * @unimod.action.descr   
	 */
	public void z7(StateMachineContext context) {
		TRACE(" .  .");
	}
	
	/**
	 * @unimod.action.descr   - 
	 */
	public void z8(StateMachineContext context) {
		TRACE(" .   .");
	}
	
	/**
	 * @unimod.action.descr    
	 */
	public void z9(StateMachineContext context) {
		TRACE(" .");
		codeAccepted = false;
	}
	
	/**
	 * @unimod.action.descr     
	 */
	public void z4(StateMachineContext context) {
		TRACE(" .  .");
		doorwaitTimer = new Timer(false);
		doorwaitTimer.schedule(new TimerTask() {
			public void run() {
				DoorlockControl.this.engine.getEventManager().handle(new Event(DL6), StateMachineContextImpl.create());
			}
		}, DOOR_OPENING_AWAITING_PERIOD);
	}	
	
	/*
	 * (non-Javadoc)
	 * @see ru.ifmo.ihouse.core.Device#handle(ru.ifmo.ihouse.core.Message)
	 */
	public boolean handle(Message msg) {
		if (msg.event.equals(DL1)) {
			Long code = msg.param1;
			if (_accCodes.contains(code)) codeAccepted = true;
			else codeAccepted = false;
		} else if (msg.event.equals(SystemEventProvider.E_ALARMOFF)) {
			if (msg.destination != id) return false;
		}
			
		return super.handle(msg);
	}
	
	/*
	 * (non-Javadoc)
	 * @see ru.ifmo.ihouse.core.Device#init()
	 */
	public void init() {
		StringTokenizer st = new StringTokenizer(accCodes, ",");
		while (st.hasMoreTokens()) {
			String s = st.nextToken();
			try {
				_accCodes.add(Long.parseLong(s));	
			} catch (NumberFormatException e) {
				TRACE("Can't parse code: " + s);
			}
		}
	}
}
