package ru.ifmo.ihouse;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.util.StringTokenizer;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Style;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.xml.DOMConfigurator;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.sun.imageio.metadata.XmlNames;

import ru.ifmo.ihouse.core.Message;
import ru.ifmo.ihouse.core.Messager;
import ru.ifmo.ihouse.events.SystemEventProvider;

/**
 *   .   .
 *    ( main),    
 *     XML-,      
 *       .
 * @author rgm
 * @version 1.0.0
 */
public class Main {

	/**
	 *          xml-,
	 *          .
	 * @param file  .
	 */
	private void LoadDeviceConfig(File file) {
		String IHOUSE_NAMESPACE		= "http://www.ifmo.ru/ihouse";
		String DEVICE_ELEMENT_NAME	= "device";

		Document document;
		try {
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
			factory.setNamespaceAware(true);
			DocumentBuilder builder = factory.newDocumentBuilder();
					
			document = builder.parse(file);
		} catch (SAXException e) {
			System.out.println("    \"" + file + "\": " + e.getMessage());
			return;
		} catch (IOException e) {
			System.out.println("    \"" + file + "\": " + e.getMessage());
			return;
		} catch (ParserConfigurationException e) {
			System.out.println("    \"" + file + "\": " + e.getMessage());
			return;
		}
				
		Element root = document.getDocumentElement();
		NodeList devices = root.getElementsByTagNameNS(IHOUSE_NAMESPACE, DEVICE_ELEMENT_NAME);
		for (int i = 0; i < devices.getLength(); i++) { //   
			Element device = (Element)devices.item(i);
			NamedNodeMap attributes = device.getAttributes();
			
			String className	= attributes.getNamedItem("class").getNodeValue();
			String deviceName	= attributes.getNamedItem("name").getNodeValue();
			String xmlName		= null;
			try {
				xmlName	= attributes.getNamedItem("xml").getNodeValue();
			} catch (NullPointerException e) { xmlName = null; };
			try {
				Class deviceClass = Class.forName(className);
				Object obj = deviceClass.newInstance();
				
				/*    */
				try {
					Field field = deviceClass.getField("name");
					field.set(obj, deviceName);
				} catch (NoSuchFieldException e) {
					System.out.println("[CONFIG] NO FIELD FOUND in class for name of device " + deviceName);
				}
				
				/*     ,    */
				NodeList deviceConfig = device.getElementsByTagName("param");
				for (int j = 0; j < deviceConfig.getLength(); j++) {
					Node param = deviceConfig.item(j);
					if (param == null || param.getAttributes() == null) continue;
					Node paramName	= param.getAttributes().getNamedItem("name");
					if (paramName == null) { //      
						System.out.println("[CONFIG]      '" + param.getNodeName() + "'  " + deviceName);
						continue;
					}
					Node paramValue = param.getAttributes().getNamedItem("value");
					if (paramValue == null) { //     
						System.out.println("[CONFIG]      '" + param.getNodeName() + "'  " + deviceName);
						continue;
					}
					String value = paramValue.getNodeValue();
					/*       */
					try {
						Field field = deviceClass.getField(paramName.getNodeValue());
						if (field.getType() == String.class) field.set(obj, value);
						else if (field.getType() == Integer.class) field.set(obj, Integer.parseInt(value));
						else if (field.getType() == Double.class) field.set(obj, Double.parseDouble(value));
						else System.out.println("[CONFIG]        '" + paramName.getNodeValue() + "'  " + deviceName);
					} catch (NoSuchFieldException e) {
						System.out.println("[CONFIG]        '" + paramName.getNodeValue() + "'  " + deviceName);
					} catch (NumberFormatException e) {
						System.out.println("[CONFIG]      '" + paramName.getNodeValue() + "'  " + deviceName);
					}
				}
				
				deviceClass.getMethod("init", null).invoke(obj, null); //      
				
				Class params[] = new Class[1];
				params[0] = String.class;
				if (xmlName != null) deviceClass.getMethod("launch", params).invoke(obj, xmlName);
				else deviceClass.getMethod("launch", null).invoke(obj, null);
				
				
			} catch (NoSuchMethodException e) {
				System.out.println("[CONFIG]  init()   " + deviceName + "  !");
			} catch (ClassNotFoundException e) {
				System.out.println("[CONFIG]  " + className + "   " + deviceName + "  !");
			} catch (IllegalAccessException e) {
				System.out.println("[CONFIG]  !");
			} catch (InvocationTargetException e) {
				System.out.println("[CONFIG]   !");
			} catch (InstantiationException e) {
				
			}
		}
	}
	
	/**
	 *    .
	 * @param args   .
	 */
	public static void main(String[] args) {
		System.setProperty("file.encoding","Cp866");
		System.setProperty("console.encoding","Cp866");
		Main m = new Main();
		m.run();
	}

	/**
	 *   ,  ,
	 *   .
	 */
	protected void run() {
		DOMConfigurator.configure(getClass().getResource("log4j.xml"));
		
		LoadDeviceConfig(new File("config.xml"));
		createUI();
		Message msg = Messager.empty();
		msg.event = SystemEventProvider.E_START;
		Messager.send(msg);
	}
	
	private final JTextPane text = new JTextPane();
	private StyledDocument sd;
	
	/**
	 *     SWING.
	 */
	protected void createUI() {
		Tracer.init(text);
		sd = text.getStyledDocument();
	    Style redStyle = text.addStyle("Red", null);
	    StyleConstants.setForeground(redStyle, Color.red);
	    Style blueStyle = text.addStyle("Blue", null);
	    StyleConstants.setForeground(blueStyle, Color.blue);
	    text.setFont(new Font("Courier New", 0, 12));
		
		JFrame main = new JFrame("IHouse console");
		main.setSize(400, 300);
		main.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		main.setLayout(new BorderLayout());
		
		JScrollPane textScroller = new JScrollPane(text);
		text.setEditable(false);
		main.add(textScroller, BorderLayout.CENTER);
		
		JPanel inputPanel = new JPanel();
		inputPanel.setLayout(new BorderLayout());
		final JTextField input = new JTextField();
		inputPanel.add(input, BorderLayout.CENTER);
		JButton send = new JButton("send");
		send.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				String str = input.getText();
				StringTokenizer st = new StringTokenizer(str);
				String command = st.nextToken();
				if (command.equals("send")) {
					try {
						Message msg = Messager.empty();
						if (st.hasMoreTokens()) msg.event = st.nextToken();
						if (st.hasMoreTokens()) msg.destination	= Integer.parseInt(st.nextToken());
						if (st.hasMoreTokens()) msg.source	= Integer.parseInt(st.nextToken());
						if (st.hasMoreTokens()) msg.param1	= Integer.parseInt(st.nextToken());
						if (st.hasMoreTokens()) msg.param2	= Integer.parseInt(st.nextToken());
						if (st.hasMoreTokens()) msg.param3	= st.nextToken();
						Messager.send(msg);
						str += " - ";
						str += " : " + msg.toString();
						System.out.println("[SEND] " + msg.toString());
					} catch (NumberFormatException err) {
						str += " -  ";
					}
				} else if (command.equals("setenv")) {
					String var = null, val = null;
					try {
						if (st.hasMoreTokens()) var = st.nextToken();
						if (st.hasMoreTokens()) val = st.nextToken();
						if (var != null) {
							Field field = Environment.class.getField(var);
							if (field.getType() == Boolean.class) {
								field.set(Environment.class, Boolean.parseBoolean(val));
							} else if (field.getType() == Integer.class) {
								field.set(Environment.class, Integer.parseInt(val));
							}
							str += " - ";
						}
					} catch (NumberFormatException err) {
						str += " -  ";
					} catch (NoSuchFieldException err) {
						str += " -  :    ";
					} catch (IllegalAccessException err) {
						str += " -  ";
					}
				} else str += " -  ";
				
				
				input.setText("");
				try {
					text.getDocument().insertString(0, "> " + str + "\n", text.getStyle("Blue"));//.setText(text.getText() + );
				} catch (BadLocationException err) {}
			}
		});
		
		inputPanel.add(send, BorderLayout.EAST);
		
		main.add(inputPanel, BorderLayout.SOUTH);
		
		//
		JPanel msgPanel = new JPanel();
		msgPanel.setLayout(new GridLayout(3, 1));
		
		JPanel eventPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JTextField event = new JTextField(20);
		JLabel eventLabel = new JLabel("Event:");
		eventPanel.add(eventLabel);
		eventPanel.add(event);
		
		JPanel destPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JLabel destLabel = new JLabel("Destination:");
		JTextField dest	 = new JTextField(20);
		destPanel.add(destLabel);
		destPanel.add(dest);
		
		JPanel sendMsgPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		JButton sendMsg	 = new JButton("send");
		sendMsgPanel.add(sendMsg);
		
		msgPanel.add(eventPanel);
		msgPanel.add(destPanel);
		msgPanel.add(sendMsgPanel);
		
		main.setVisible(true);
	}
	
	public void init() {
		Main m = new Main();
		m.run();
	}
}
