package ru.ifmo.ihouse.core;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.ConcurrentLinkedQueue;

import org.apache.commons.logging.LogFactory;

import ru.ifmo.ihouse.Tracer;
import ru.ifmo.ihouse.events.SystemEventProvider;

import com.evelopers.common.exception.CommonException;
import com.evelopers.unimod.adapter.standalone.Run;
import com.evelopers.unimod.core.stateworks.Event;
import com.evelopers.unimod.core.stateworks.Model;
import com.evelopers.unimod.debug.ExceptionHandlerImpl;
import com.evelopers.unimod.runtime.ControlledObject;
import com.evelopers.unimod.runtime.ControlledObjectsMap;
import com.evelopers.unimod.runtime.EventProcessorListener;
import com.evelopers.unimod.runtime.ExceptionHandler;
import com.evelopers.unimod.runtime.ModelEngine;
import com.evelopers.unimod.runtime.context.StateMachineContext;
import com.evelopers.unimod.runtime.context.StateMachineContextImpl;
import com.evelopers.unimod.runtime.interpretation.InterpretationHelper;
import com.evelopers.unimod.runtime.logger.SimpleLogger;
import com.evelopers.unimod.transform.TransformException;
import com.evelopers.unimod.transform.xml.XMLToModel;

/**
 *      . 
 *     ,  
 *        .
 * @author rgm
 * @version 1.5.0
 */
public class Device implements Messager.IReciever, ControlledObject {
	
	/**
	 * 
	 */
	public Device() {
		id = DeviceMaster.getIDFor(this);
		Messager.instance().addReciever(this);
	}
	
	public void launch(String xml) {
		InitAutomata(xml, this);
		start();
	}
	public void launch() {
		System.out.println("[INFO]  : " + this.toString() + "   xml- !");
	};
	
	/**
	 *    
	 */
	protected ModelEngine engine;
	
	/** @deprecated */
	private int heartbeatTime = 0;
	/** @deprecated */
	public static final int HEARTBEAT_PERIOD = 10000;
	
	/**
	 * id 
	 */
	protected long id;
	
	/**
	 *  .    .
	 */
	public String name;
	
	/**
	 *  id .
	 * @return id 
	 */
	public long id() { return id; }
	
	/**
	 *       ,  
	 *     
	 * .
	 */
	public void init() {};
	
	/**
	 *    .
	 * @param xmlmodel     xml- 
	 * @param device_co  .
	 */
	protected void InitAutomata(String xmlmodel, ControlledObject device_co) {
		/*    xml- */
		Model model;
		try {
			InputStream is = new FileInputStream(xmlmodel);
			if (is == null) {
				System.out.println("is == null");
				System.exit(0);
			}
			model = XMLToModel.loadAndCompile(is);
		} catch (IOException e) {
			System.out.println("[ERROR]    xml-: " + xmlmodel);
			throw new RuntimeException();
		} catch (TransformException e) {
			System.err.println(e.getMessage());
			throw new RuntimeException();
		}

		/*    */
		InterpretationHelper helper = InterpretationHelper.getInstance();  
		try {
			engine = helper.createBuildInModelEngine(model, new DeviceCOMap(device_co),true);
		} catch (CommonException e) {
			System.err.println(e.getMessage());
			throw new RuntimeException();
		}

		
		EventProcessorListener logger = new SimpleLogger(LogFactory.getLog(Run.class));
		engine.getEventProcessor().addEventProcessorListener(logger);
		ExceptionHandler eh = new ExceptionHandlerImpl();
		engine.getEventProcessor().addExceptionHandler(eh);	
	}

	/**
	 *  .
	 */
	private ConcurrentLinkedQueue<Message> messages = new ConcurrentLinkedQueue<Message>();
	
	/**
	 *        .
	 * @see ru.ifmo.ihouse.core.Messager.IReciever#recieve(ru.ifmo.ihouse.core.Message)
	 */
	public void recieve(Message msg) {
		if (msg != null) messages.add(msg); 
	}
	
	private static class DeviceCOMap implements ControlledObjectsMap {
		private ControlledObject co;
			
		public DeviceCOMap(ControlledObject co) {
			this.co = co;
		}
		public ControlledObject getControlledObject(String coName) {
			return co;
		}
	}
	
	/**
	 *   .
	 */
	private Timer timer;
	
	private final static StateMachineContext ec = StateMachineContextImpl.create();
	
	/**
	 *         .
	 */
	protected void start() {
		try {
			if (engine != null) engine.start();
		} catch (CommonException e) {
			TRACE("ERROR: Can't start device automata!");
		}
		
		timer = new Timer(false);
		timer.schedule(
                new TimerTask() {
                    public void run() {
                    		/*
        					if ((heartbeatTime += 1000) >= HEARTBEAT_PERIOD) {
        						heartbeatTime = 0;
        						Message msg = Messager.empty();
        						msg.event = SystemEventProvider.E_HEARTBEAT;
        						Messager.send(msg);
        					}
        					*/
        					Message msg = messages.poll();
        					if (msg != null) {
        						/* translating messages to automata */
        						if (engine != null && handle(msg)) {
	        						if (msg.event != null) Device.this.engine.getEventManager().handleAndWait(new Event(msg.event), StateMachineContextImpl.create());
	        						else Device.this.engine.getEventManager().handleAndWait(new Event(SystemEventProvider.E2), ec);
	        					}
        						/* handling system messages */
	        					if (msg.event != null && msg.event.equals(SystemEventProvider.E_TERMINATE)) { 
	        						System.out.println("[EXIT (" + SystemEventProvider.E_TERMINATE + ")] " + name + " is OFFLINE");
	        						timer.cancel();
	        					}
                    		}
                    }
                }, 100, 500);
	}
	
	/**
	 *    .
	 * @param msg  .
	 * @return TRUE,     .
	 */
	protected boolean handle(Message msg) {
		if (msg.event == SystemEventProvider.E_HEARTBEAT) return false; //   HEARTBEAT
		return true;
	}
	
	/**
	 *     .
	 * @param message     .
	 */
	protected void TRACE(String message) {
		Tracer.TRACE("[TRACE] " + "(" + id + ") '" + name + "' reported: " + message);
	}
	
	/**
	 *     .
	 * @param message     .
	 */
	protected void ALARM(String message) {
		Tracer.ALARM("[ALERT] " + "(" + id + ") '" + name + "' reported: " + message);
	}
	
	/**
	 * 
	 */
	public void dispose() {
		timer.cancel();
	}
}
